@@ -1,8 +1,10 @@
-from typing import Dict, Optional, Set, cast
+import secrets
+from re import Pattern
+from typing import Dict, List, Optional, Set, cast
 
 from itsdangerous import BadSignature
 from itsdangerous.url_safe import URLSafeSerializer
-from passlib.pwd import genword
+from starlette.datastructures import URL
 from starlette.middleware.base import BaseHTTPMiddleware, RequestResponseEndpoint
 from starlette.requests import Request
 from starlette.responses import PlainTextResponse
@@ -23,6 +25,7 @@ class CSRFMiddleware(BaseHTTPMiddleware):
         cookie_samesite: str = "lax",
         header_name: str = "x-csrftoken",
         safe_methods: Set[str] = {"GET", "HEAD", "OPTIONS", "TRACE"},
+        exempt_urls: Optional[List[Pattern]] = None,
     ) -> None:
         super().__init__(app)
         self.serializer = URLSafeSerializer(secret, "csrftoken")
@@ -36,12 +39,15 @@ class CSRFMiddleware(BaseHTTPMiddleware):
         self.cookie_samesite = cookie_samesite
         self.header_name = header_name
         self.safe_methods = safe_methods
+        self.exempt_urls = exempt_urls
 
     async def dispatch(self, request: Request, call_next: RequestResponseEndpoint):
         csrf_cookie = request.cookies.get(self.cookie_name)
 
-        if request.method not in self.safe_methods and self._has_sensitive_cookies(
-            request.cookies
+        if (
+            request.method not in self.safe_methods
+            and self._has_sensitive_cookies(request.cookies)
+            and not self._url_is_exempt(request.url)
         ):
             csrf_header = request.headers.get(self.header_name)
             if (
@@ -76,13 +82,21 @@ class CSRFMiddleware(BaseHTTPMiddleware):
                 return True
         return False
 
+    def _url_is_exempt(self, url: URL) -> bool:
+        if not self.exempt_urls:
+            return False
+        for exempt_url in self.exempt_urls:
+            if exempt_url.match(url.path):
+                return True
+        return False
+
     def _generate_csrf_token(self) -> str:
-        return cast(str, self.serializer.dumps(genword(entropy="strong")))
+        return cast(str, self.serializer.dumps(secrets.token_urlsafe(128)))
 
     def _csrf_tokens_match(self, token1: str, token2: str) -> bool:
         try:
             decoded1: str = self.serializer.loads(token1)
             decoded2: str = self.serializer.loads(token2)
-            return decoded1 == decoded2
+            return secrets.compare_digest(decoded1, decoded2)
         except BadSignature:
             return False
